!*****************************************************************************!
!* 
!*      $RCSfile: Euler2D.f90,v $
!*
!*      $Revision: 1.13 $
!*
!*      $Date: 2004/03/08 17:50:13 $
!*
!*      $Author: roller $
!*
!*****************************************************************************!

PROGRAM Euler2D
  USE SetBoundaryCondition_mod
  USE SetBoundaryConditionLR_mod
  USE CalcTimeStep_mod
  USE CloseEuler2D_mod
  USE CreateMesh_mod
  USE DataOutput_mod
  USE ExactSolution_mod
  USE IniEuler2D_mod
  USE InitialCondition_mod
  USE PrimCons_mod
  USE TimeDisc_mod
  USE Types
  !---------------------------------------------------------------------------!
  IMPLICIT NONE
  !---------------------------------------------------------------------------!
  TYPE(tConstants)            :: Const              ! Constants structure     !
  TYPE(tMesh)                 :: Mesh               ! Mesh structure          !
  TYPE(tFileIO)               :: FileIO             ! FileIO structure        !
  TYPE(tMPI)                  :: MPI                ! MPI data                !
  REAL, POINTER               :: cvar(:,:,:)        ! Conservative Variables  !
  REAL, POINTER               :: pvar(:,:,:)        ! Primitive Variables     !
  REAL                        :: time               ! Simulation-time         !
  REAL                        :: printtime          ! Next printtime          !
  REAL                        :: dt                 ! Timestep                !
  INTEGER                     :: iter               ! Iteration number        !
  !---------------------------------------------------------------------------!

  CALL IniEuler2D(pvar,cvar,iter,dt,time, &       ! Initialization of ...     !
       printtime,Const, Mesh, FileIO, MPI)        ! ... simulation parameters !
  CALL CreateMesh(Const, Mesh, MPI)               ! Mesh geometry             !
  CALL InitialCondition(pvar,Const,Mesh,MPI)      ! Initial condition         !
  CALL PrimCons(2,pvar,cvar,Const,Mesh)           ! Generate cons. variables  !
  CALL iniCalcTimeStep(Mesh)                      ! Initialize CalcTimeStep   !
  CALL iniBoundaryCondition(Const,Mesh)           ! Initialize SetBoundaryCond!
  CALL iniBoundaryConditionLR(Const,Mesh)         ! Initialize SetBoundaryCond!
  CALL iniTimeDisc(Const,Mesh)                    ! Initialize TimeDisc module!
  CALL DataOutput(iter,dt,time,printtime,pvar, & 
       Const,Mesh,FileIO,MPI)                     ! Write initial condition   !

  SELECT CASE(Const%icType)                       ! For 1D Riemann problems:  !
  CASE(5,6,7,8,9)                                 ! Initialize ...            !
     call IniExactSolution(pvar,Const,Mesh,FileIO,MPI)! ...  ExactSolution module !
  CASE DEFAULT
  END SELECT

  DO iter = 1, Const%MaxIter
     ! Calculate dt=CFL*l/|u+a|  !
!!$     if (MPI%myrank ==0) write(*,*) iter
     CALL CalcTimeStep(dt,time,printtime,pvar,Const,Mesh,MPI)
     CALL TimeDisc(iter,dt,pvar,cvar,Const,Mesh,MPI)! Time discretization       !
     CALL PrimCons(1,pvar,cvar,Const,Mesh)        ! Generate primitive vars.  !

     IF(time.EQ.printtime) THEN
        CALL DataOutput(iter,dt,time,printtime,pvar, &
             Const,Mesh, FileIO, MPI)             ! Dataoutput                !
        SELECT CASE(Const%icType)                 ! For 1D Riemann problems:  !
        CASE(5,6,7,8,9)                           ! Calculate Exact Solution  !
           call ExactSolution(iter,dt,time,Const,Mesh,FileIO,MPI)
        CASE DEFAULT
        END SELECT
     ENDIF

     IF(time.GE.Const%Stoptime) THEN
        WRITE(*,*) 'Time limit reached. Stopping calculation! '
        EXIT
     ENDIF
  ENDDO

  SELECT CASE(Const%icType)                       ! For 1D Riemann problems:  !
  CASE(5,6,7,8,9)                                 ! Close ...                 !
     call CloseExactSolution()                    ! ...  ExactSolution module !
  CASE DEFAULT
  END SELECT

  call closeBoundaryCondition()
  call closeBoundaryConditionLR()
  CALL closeCalcTimeStep()                        ! Close TimeStep module     !
  CALL closeTimeDisc()                            ! Close TimeDisc module     !
  CALL CloseEuler2D(pvar,cvar,Mesh,MPI)           ! Close Program             !

END PROGRAM Euler2D
