!*****************************************************************************!
!* 
!*      $RCSfile: CalcTimeStep.F90,v $
!*
!*      $Revision: 1.8 $
!*
!*      $Date: 2004/02/09 12:13:53 $
!*
!*      $Author: roller $
!*
!*****************************************************************************!

MODULE CalcTimeStep_mod
  !---------------------------------------------------------------------------!
  IMPLICIT NONE
  PRIVATE
  !---------------------------------------------------------------------------!
  INTERFACE iniCalcTimeStep
     MODULE PROCEDURE iniCalcTimeStep
  END INTERFACE
  INTERFACE closeCalcTimeStep
     MODULE PROCEDURE closeCalcTimeStep
  END INTERFACE
  INTERFACE CalcTimeStep
     MODULE PROCEDURE CalcTimeStep
  END INTERFACE
  !---------------------------------------------------------------------------!
  REAL, POINTER :: velocity(:,:)
  REAL, POINTER :: csound(:,:)
  !---------------------------------------------------------------------------!
  PUBLIC  :: iniCalcTimestep, closeCalcTimeStep, CalcTimeStep
  !---------------------------------------------------------------------------!

CONTAINS

  SUBROUTINE iniCalcTimeStep(Mesh)
    USE Types
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!
    ! Argument list declaration
    TYPE(tMesh)             :: Mesh
    ! Local variable declaration
    INTEGER                 :: allocStat
    !---------------------------------------------------------------------------!
    
    ALLOCATE(velocity(0:Mesh%IMAX+1,0:Mesh%JMAX+1), &
         csound(0:Mesh%IMAX+1,0:Mesh%JMAX+1), &
         STAT = allocStat )

    IF(allocStat.NE.0) THEN
       PRINT *, 'ERROR iniCalcTimeStep: Could not allocate all variables. '
       STOP
    ENDIF

  END SUBROUTINE iniCalcTimeStep

  SUBROUTINE closeCalcTimeStep()
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!
    
    DEALLOCATE(velocity, csound)

  END SUBROUTINE closeCalcTimeStep
  

  SUBROUTINE CalcTimeStep(dt,time,printtime,pvar,Const,Mesh,MPI)
    USE Types
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!
#ifdef parallel
    include 'mpif.h'
#endif
    !---------------------------------------------------------------------------!
    ! Argument list declaration
    TYPE(tConstants)        :: Const
    TYPE(tMesh)             :: Mesh
    TYPE(tMPI)              :: MPI
    REAL                    :: dt,dt_all
    REAL                    :: time
    REAL                    :: printtime
    REAL                    :: pvar(0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    ! Local variable declaration
    REAL                    :: vc
    !---------------------------------------------------------------------------!
    INTENT(IN)    :: printtime, pvar, Const, Mesh, MPI
    INTENT(OUT)   :: dt
    INTENT(INOUT) :: time
    !---------------------------------------------------------------------------!

    ! Calculate maximum eigenvalue in the computational domain
    velocity(:,:) = SQRT( pvar(:,:,2)*pvar(:,:,2) + pvar(:,:,3)*pvar(:,:,3) ) 
    csound(:,:) = SQRT(Const%gamma * pvar(:,:,4) / pvar(:,:,1) )
    vc = MAXVAL(velocity(:,:) + csound(:,:))

    ! Calculate timestep from CFL-condition
    dt = Const%CFL * Mesh%lmin / vc

    ! Special treatment for data output
    if ((time+dt).gt.printtime) then
       dt=printtime - time
    elseif ((time+1.5*dt).gt.printtime) then
       dt=0.5*(printtime - time)
    endif

    if ((time+dt).gt.Const%Stoptime) dt=Const%Stoptime - time
   
#ifdef parallel
    call mpi_allreduce(dt,dt_all,1,mpi_real,mpi_min,MPI%comm_cart,MPI%ierror)
    dt=dt_all
#endif

    time = time + dt

  END SUBROUTINE CalcTimeStep


END MODULE CalcTimeStep_mod
