!*****************************************************************************!
!* 
!*      $RCSfile: ComputeGradients.f90,v $
!*
!*      $Revision: 1.8 $
!*
!*      $Date: 2004/01/29 09:57:01 $
!*
!*      $Author: iagmidu $
!*
!*****************************************************************************!

MODULE ComputeGradients_mod
  !---------------------------------------------------------------------------!
  IMPLICIT NONE
  PRIVATE
  !---------------------------------------------------------------------------!
  INTERFACE ComputeGradients
     MODULE PROCEDURE ComputeGradients
  END INTERFACE
  !---------------------------------------------------------------------------!
  PUBLIC  :: ComputeGradients
  !---------------------------------------------------------------------------!

CONTAINS

  SUBROUTINE ComputeGradients(xGradient,yGradient,pvar,Const,Mesh)
    USE Types
    USE Limiter_mod
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!
    ! Argument list declaration
    TYPE(tConstants)        :: Const
    TYPE(tMesh)             :: Mesh
    REAL                    :: pvar(0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    REAL                    :: xGradient(0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    REAL                    :: yGradient(0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    ! Local variable declaration
    INTEGER                 :: i,j
    REAL                    :: slope1(Const%nvar), slope2(Const%nvar)
    REAL                    :: limslope(Const%nvar)
    !---------------------------------------------------------------------------!
    INTENT(IN)              :: pvar, Const, Mesh
    INTENT(OUT)             :: xGradient, yGradient
    !---------------------------------------------------------------------------!

    DO i=1,Mesh%IMAX
       DO j=1,Mesh%JMAX
          slope1(:) = pvar(i  ,j  ,:) - pvar(i-1,j  ,:)
          slope2(:) = pvar(i+1,j  ,:) - pvar(i  ,j  ,:)
          
          call minmod(slope1,slope2,limslope,Const)
          xGradient(i,j,:) = 1./Mesh%dx * limslope(:)

          slope1(:) = pvar(i  ,j  ,:) - pvar(i  ,j-1,:)
          slope2(:) = pvar(i  ,j+1,:) - pvar(i  ,j  ,:)


          call minmod(slope1,slope2,limslope,Const)
          yGradient(i,j,:) = 1./Mesh%dy * limslope(:)

       ENDDO
    ENDDO

    IF ((Const%BoundCond.eq.3).or.(Const%BoundCond.eq.4)) THEN
       !                                        Periodicity in x-Direction
       xGradient(0,:,:)           = xGradient(Mesh%IMAX,:,:)
       xGradient(Mesh%IMAX+1,:,:) = xGradient(1,:,:)
       yGradient(0,:,:)           = yGradient(Mesh%IMAX,:,:)
       yGradient(Mesh%IMAX+1,:,:) = yGradient(1,:,:)
    ELSEIF ((Const%BoundCond.eq.2).or.(Const%BoundCond.eq.4)) THEN
       !                                        Periodicity in y-Direction
       xGradient(:,0,:)           = xGradient(:,Mesh%JMAX,:)
       xGradient(:,Mesh%JMAX+1,:) = xGradient(:,1,:)
       yGradient(:,0,:)           = yGradient(:,Mesh%JMAX,:)
       yGradient(:,Mesh%JMAX+1,:) = yGradient(:,1,:)
    ENDIF

    
  END SUBROUTINE ComputeGradients


END MODULE ComputeGradients_mod
