!*****************************************************************************!
!* 
!*      $RCSfile: Flux_godunov.f90,v $
!*
!*      $Revision: 1.1.1.1 $
!*
!*      $Date: 2002/09/24 10:28:55 $
!*
!*      $Author: roller $
!*
!*****************************************************************************!

MODULE flux_godunov_mod
  !--------------------------------------------------------------------------
  IMPLICIT NONE
  PRIVATE
  !----------------------------------------------------------------------------
  INTERFACE flux_godunov
     MODULE PROCEDURE flux_godunov
  END INTERFACE
  !----------------------------------------------------------------------------

  PUBLIC  :: flux_godunov

CONTAINS

  SUBROUTINE flux_godunov( gamma,                     &
                           rho_l, rho_r, vn_l , vn_r, &
                           vt_l , vt_r , p_l  , p_r , &
                           flux_side                   )

    !--------------------------------------------------------------------------
    USE exact_riemann_mod, ONLY : exact_riemann 
    !--------------------------------------------------------------------------
    IMPLICIT NONE


    !argument list declaration
    REAL         :: gamma
    REAL         :: rho_l, rho_r
    REAL         :: vn_l , vn_r
    REAL         :: vt_l , vt_r
    REAL         :: p_l  , p_r
    REAL         :: flux_side(4)

    !local variable declaration
    REAL         :: c_l,c_r
    REAL         :: rho, u, p, v
    !--------------------------------------------------------------------------
    INTENT(IN)   :: gamma,                     &
                    rho_l, rho_r, vn_l , vn_r, &
                    vt_l , vt_r , p_l  , p_r 
    INTENT(OUT)  :: flux_side    
    !--------------------------------------------------------------------------
    c_l = SQRT(gamma*p_l/rho_l)
    c_r = SQRT(gamma*p_r/rho_r)

    CALL exact_riemann(gamma,rho_l,rho_r,rho  ,   &
                       vn_l ,vn_r ,u    ,   &
                       p_l  ,p_r  ,p    ,   &
                       c_l  ,c_r  ,0.0      )
    !
    !    compute interface flux from riemann solution
    !

    IF(u.GE.0.0) THEN
       v = vt_l
    ELSE
       v = vt_r
    ENDIF

    flux_side(1) = rho*u
    flux_side(2) = rho*u*u + p
    flux_side(3) = rho*u*v
    flux_side(4) = u*(gamma/(gamma-1.0)*p+0.5*rho*(u*u+v*v))


    RETURN
  END SUBROUTINE flux_godunov

END MODULE flux_godunov_mod
