!*****************************************************************************!
!* 
!*      $RCSfile: Flux_lax_friedrichs.f90,v $
!*
!*      $Revision: 1.1 $
!*
!*      $Date: 2004/01/29 09:57:48 $
!*
!*      $Author: iagmidu $
!*
!*****************************************************************************!

MODULE flux_lax_friedrichs_mod
  !--------------------------------------------------------------------------
  IMPLICIT NONE
  PRIVATE
  !----------------------------------------------------------------------------
  INTERFACE flux_lax_friedrichs
     MODULE PROCEDURE flux_lax_friedrichs
  END INTERFACE
  !----------------------------------------------------------------------------

  PUBLIC  :: flux_lax_friedrichs

CONTAINS

  SUBROUTINE flux_lax_friedrichs( gamma,  dx, dt,     &
       rho_l, rho_r, vn_l , vn_r, &
       vt_l , vt_r , p_l  , p_r , &
       flux_side                   )
    !--------------------------------------------------------------------------
    IMPLICIT NONE
    !--------------------------------------------------------------------------
    !argument list declaration
    REAL         :: gamma, dx, dt
    REAL         :: rho_l, rho_r
    REAL         :: vn_l , vn_r
    REAL         :: vt_l , vt_r
    REAL         :: p_l  , p_r
    REAL         :: flux_side(4)
    !local variable declaration
    REAL         :: e_r,e_l 
    REAL         :: a
    REAL         :: delta_u(4)
    REAL         :: f1_r(4),f1_l(4)
    !--------------------------------------------------------------------------
    INTENT(IN)   :: gamma, rho_l, rho_r, vn_l, vn_r, vt_l, vt_r, p_l, p_r 
    INTENT(OUT)  :: flux_side    
    !--------------------------------------------------------------------------

    ! Calculate left/right energy and enthalpy

    e_r=1./(gamma-1.)*p_r+0.5*rho_r*(vn_r*vn_r+vt_r*vt_r)
    e_l=1./(gamma-1.)*p_l+0.5*rho_l*(vn_l*vn_l+vt_l*vt_l)

    ! Calculate the differences of the conservative variables

    delta_u(1)  = rho_r      - rho_l
    delta_u(2)  = rho_r*vn_r - rho_l*vn_l
    delta_u(3)  = rho_r*vt_r - rho_l*vt_l
    delta_u(4)  = e_r        - e_l

    ! Calculalte the physical fluxes

    f1_r(1) = rho_r*vn_r
    f1_r(2) = rho_r*vn_r*vn_r + p_r
    f1_r(3) = rho_r*vn_r*vt_r
    f1_r(4) = vn_r*(e_r + p_r)

    f1_l(1) = rho_l*vn_l
    f1_l(2) = rho_l*vn_l*vn_l + p_l
    f1_l(3) = rho_l*vn_l*vt_l
    f1_l(4) = vn_l*(e_l + p_l)

    ! For the local Lax-Friedrichs flux use a instead of dx/dt
    ! a = MAX( abs(vn_l)+sqrt(gamma*p_l/rho_l) , abs(vn_r)+sqrt(gamma*p_r/rho_r) ) 

    ! Calculate Lax-Friedrichs flux

    flux_side(:) = 0.5*( f1_r(:)+f1_l(:) ) - 0.5*dx/dt*delta_u(:) 

    RETURN
  END SUBROUTINE flux_lax_friedrichs

END MODULE flux_lax_friedrichs_mod
