!*****************************************************************************!
!* 
!*      $RCSfile: Flux_steger_warming.f90,v $
!*
!*      $Revision: 1.2 $
!*
!*      $Date: 2003/01/13 16:48:44 $
!*
!*      $Author: iagmidu $
!*
!*****************************************************************************!

MODULE flux_steger_warming_mod
  !--------------------------------------------------------------------------
  IMPLICIT NONE
  PRIVATE
  !----------------------------------------------------------------------------
  INTERFACE flux_steger_warming
     MODULE PROCEDURE flux_steger_warming
  END INTERFACE
  !----------------------------------------------------------------------------

  PUBLIC  :: flux_steger_warming

CONTAINS

  SUBROUTINE flux_steger_warming( gamma,                     &
                           rho_l, rho_r, vn_l , vn_r, &
                           vt_l , vt_r , p_l  , p_r , &
                           flux_side                   )

    !--------------------------------------------------------------------------
    IMPLICIT NONE
    !--------------------------------------------------------------------------

    !argument list declaration
    REAL         :: gamma
    REAL         :: rho_l, rho_r
    REAL         :: vn_l , vn_r
    REAL         :: vt_l , vt_r
    REAL         :: p_l  , p_r
    REAL         :: flux_side(4)

    !local variable declaration
    REAL         :: c_l,c_r
    REAL         :: ap(4), am(4), a_l(4), a_r(4)
    REAL         :: fp(4), fm(4)
    INTEGER      :: i
    !--------------------------------------------------------------------------
    INTENT(IN)   :: gamma,                     &
                    rho_l, rho_r, vn_l , vn_r, &
                    vt_l , vt_r , p_l  , p_r 
    INTENT(OUT)  :: flux_side    
    !--------------------------------------------------------------------------
    ! Calculation of sound speeds
    c_l=SQRT(gamma*p_l/rho_l)
    c_r=SQRT(gamma*p_r/rho_r)
    ! Calculation of eigenvalues
    a_l(1) = vn_l-c_l
    a_l(2) = vn_l
    a_l(3) = vn_l
    a_l(4) = vn_l+c_l
    a_r(1) = vn_r-c_r
    a_r(2) = vn_r
    a_r(3) = vn_r
    a_r(4) = vn_r+c_r
    ! Calculation of positive and negative eigenvalues
    DO i=1,4
       ap(i) = max(a_l(i),0.)
       am(i) = min(a_r(i),0.)
    ENDDO
    ! Positive flux from left to right
    fp(1)=rho_l/2./gamma*(2.*(gamma-1.)*ap(2)+ap(1)+ap(4))
    fp(2)=fp(1)*vn_l+(ap(4)-ap(1))*rho_l*c_l/2./gamma
    fp(3)=fp(1)*vt_l
    fp(4)=fp(1)*0.5*(vn_l*vn_l+vt_l*vt_l)+(ap(4)-ap(1))*rho_l*c_l*vn_l/2./gamma &
         +(ap(4)+ap(1))*rho_l*c_l*c_l/2./gamma/(gamma-1.)
    ! Negative flux from right to left
    fm(1)=rho_r/2./gamma*(2.*(gamma-1.)*am(2)+am(1)+am(4))
    fm(2)=fm(1)*vn_r+(am(4)-am(1))*rho_r*c_r/2./gamma
    fm(3)=fm(1)*vt_r
    fm(4)=fm(1)*0.5*(vn_r*vn_r+vt_r*vt_r)+(am(4)-am(1))*rho_r*c_r*vn_r/2./gamma &
         +(am(4)+am(1))*rho_r*c_r*c_r/2./gamma/(gamma-1.)
    ! Final flux
    flux_side(:)=fp(:) + fm(:)
    RETURN
  END SUBROUTINE flux_steger_warming

END MODULE flux_steger_warming_mod
