!*****************************************************************************!
!* 
!*      $RCSfile: Reconstruction.f90,v $
!*
!*      $Revision: 1.10 $
!*
!*      $Date: 2004/02/09 11:57:22 $
!*
!*      $Author: roller $
!*
!*****************************************************************************!

MODULE Reconstruction_mod
  !---------------------------------------------------------------------------!
  IMPLICIT NONE
  PRIVATE
  !---------------------------------------------------------------------------!
  INTERFACE Reconstruction
     MODULE PROCEDURE Reconstruction
  END INTERFACE
  !---------------------------------------------------------------------------!
  PUBLIC  :: Reconstruction
  !---------------------------------------------------------------------------!

CONTAINS

  SUBROUTINE Reconstruction(dt,xPrimLR,yPrimLR,xGradient,yGradient,pvar,Const,Mesh,MPI)
    USE Types
    USE SetBoundaryConditionLR_mod
    USE PrimConsLR_mod
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!
    ! Argument list declaration
    TYPE(tConstants)        :: Const
    TYPE(tMesh)             :: Mesh
    TYPE(tMPI)              :: MPI
    REAL                    :: dt
    REAL                    :: pvar(0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    REAL                    :: xGradient(0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    REAL                    :: yGradient(0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    REAL                    :: xPrimLR(2,0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    REAL                    :: yPrimLR(2,0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    ! Local variable declaration
    REAL                    :: fx(2,0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    REAL                    :: fy(2,0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    REAL                    :: xConsLR(2,0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    REAL                    :: yConsLR(2,0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    REAL                    :: gamm1, gamma1q
    !---------------------------------------------------------------------------!
    INTENT(IN)              :: xGradient, yGradient, pvar, Const, Mesh
    INTENT(OUT)             :: xPrimLR, yPrimLR
    !---------------------------------------------------------------------------!

    gamm1   = Const%gamma - 1.0
    gamma1q = 1.0 / gamm1

    SELECT CASE(Const%SpaceOrder)
    CASE(1)  ! First order spatial discretisation

       ! Reconstruction in x direction
       xPrimLR(1,:,:,:) = pvar(:,:,:)                                ! Left state
       xPrimLR(2,:,:,:) = pvar(:,:,:)                                ! Right state

       ! Reconstruction in y direction
       yPrimLR(1,:,:,:) = pvar(:,:,:)                                ! Left state
       yPrimLR(2,:,:,:) = pvar(:,:,:)                                ! Right state


    CASE(2)  ! Second order spatial discretisation

       ! Reconstruction in x direction
       xPrimLR(1,:,:,:) = pvar(:,:,:)  - Mesh%dx/2.*xGradient(:,:,:) ! Left state
       xPrimLR(2,:,:,:) = pvar(:,:,:)  + Mesh%dx/2.*xGradient(:,:,:) ! Right state

       ! Reconstruction in y direction
       yPrimLR(1,:,:,:) = pvar(:,:,:)  - Mesh%dy/2.*yGradient(:,:,:) ! Left state
       yPrimLR(2,:,:,:) = pvar(:,:,:)  + Mesh%dy/2.*yGradient(:,:,:) ! Right state

    CASE DEFAULT
       PRINT *, 'ERROR Reconstruction. SpaceOrder must be 1 or 2'
       STOP
    END SELECT
       
    call SetBoundaryConditionLR(xPrimLR,Const,Mesh,MPI)
    call SetBoundaryConditionLR(yPrimLR,Const,Mesh,MPI)
    
    SELECT CASE(Const%TimeOrder)
    CASE(1)  ! First order time discretisation

       ! Trivial: xPrimLR=xPrimLR, yPrimLR=yPrimLR

    CASE(2)  ! Second order time discretisation

       ! transform to conservative variables
       call PrimconsLR(2,xPrimLR,xConsLR,Const,Mesh)
       call PrimconsLR(2,yPrimLR,yConsLR,Const,Mesh)

       !Fluxes of the +/- Prims:

       !x-direction: fx(U_i-)
       fx(:,:,:,1)= xConsLR(:,:,:,2)                                          ! rho*u
       fx(:,:,:,2)= xConsLR(:,:,:,2)*xPrimLR(:,:,:,2) + xPrimLR(:,:,:,4)    ! rho*u^2 +p
       fx(:,:,:,3)= xConsLR(:,:,:,2)*xPrimLR(:,:,:,3)                        ! rho*u*v
       fx(:,:,:,4)= xPrimLR(:,:,:,2)*( xConsLR(:,:,:,4) + xPrimLR(:,:,:,4) )! u*(e+p)
       
       fy(:,:,:,1)= yConsLR(:,:,:,3)                                          ! rho*v
       fy(:,:,:,2)= yConsLR(:,:,:,2)*yPrimLR(:,:,:,3)                        ! rho*u*v
       fy(:,:,:,3)= yConsLR(:,:,:,3)*yPrimLR(:,:,:,3) + yPrimLR(:,:,:,4)    ! rho*v^2 +p
       fy(:,:,:,4)= yPrimLR(:,:,:,3)*( yConsLR(:,:,:,4) + yPrimLR(:,:,:,4) )! v*(e+p)
       
       ! Reconstruction in time:

       ! Reconstruction in x direction
       xConsLR(1,:,:,:) = xConsLR(1,:,:,:)  + &                     ! Left state
            dt/(2.*Mesh%dx)*( fx(1,:,:,:) - fx(2,:,:,:) ) + &
            dt/(2.*Mesh%dy)*( fy(1,:,:,:) - fy(2,:,:,:) )   
       xConsLR(2,:,:,:) = xConsLR(2,:,:,:)  + &                     ! Right state
            dt/(2.*Mesh%dx)*( fx(1,:,:,:) - fx(2,:,:,:) ) + &
            dt/(2.*Mesh%dy)*( fy(1,:,:,:) - fy(2,:,:,:) )   

       ! Reconstruction in y direction
       yConsLR(1,:,:,:) = yConsLR(1,:,:,:)  + &                     ! Left state
            dt/(2.*Mesh%dx)*( fx(1,:,:,:) - fx(2,:,:,:) ) + &
            dt/(2.*Mesh%dy)*( fy(1,:,:,:) - fy(2,:,:,:) )   
       yConsLR(2,:,:,:) = yConsLR(2,:,:,:)  + &                     ! Right state
            dt/(2.*Mesh%dx)*( fx(1,:,:,:) - fx(2,:,:,:) ) + &
            dt/(2.*Mesh%dy)*( fy(1,:,:,:) - fy(2,:,:,:) ) 

       ! transform back to primitive variables

       call PrimconsLR(1,xPrimLR,xConsLR,Const,Mesh)
       call PrimconsLR(1,yPrimLR,yConsLR,Const,Mesh)
       
    CASE DEFAULT
       PRINT *, 'ERROR Reconstruction. TimeOrder must be 1 or 2'
       STOP
    END SELECT
       
    call SetBoundaryConditionLR(xPrimLR,Const,Mesh,MPI)
    call SetBoundaryConditionLR(yPrimLR,Const,Mesh,MPI)
    
  END SUBROUTINE Reconstruction


END MODULE Reconstruction_mod
