!*****************************************************************************!
!* 
!*      $RCSfile: TimeDisc.f90,v $
!*
!*      $Revision: 1.9 $
!*
!*      $Date: 2004/02/09 11:57:22 $
!*
!*      $Author: roller $
!*
!*****************************************************************************!

MODULE TimeDisc_mod
  !---------------------------------------------------------------------------!
  IMPLICIT NONE
  PRIVATE
  !---------------------------------------------------------------------------!
  INTERFACE TimeDisc
     MODULE PROCEDURE TimeDisc
  END INTERFACE
  INTERFACE IniTimeDisc
     MODULE PROCEDURE IniTimeDisc
  END INTERFACE
  INTERFACE CloseTimeDisc
     MODULE PROCEDURE CloseTimeDisc
  END INTERFACE

  REAL, POINTER      :: k1(:,:,:)              ! k1 for 2nd order Runge-Kutta !
  REAL, POINTER      :: k2(:,:,:)              ! k2 for 2nd order Runge-Kutta !
  REAL, POINTER      :: new_cvar(:,:,:)        ! Field at timestep n+1        !
  REAL, POINTER      :: xGradient(:,:,:)       ! Gradient in x direction      !
  REAL, POINTER      :: yGradient(:,:,:)       ! Gradient in y direction      !
  REAL, POINTER      :: xPrimLR(:,:,:,:)      ! Left and right states for RP !
  REAL, POINTER      :: yPrimLR(:,:,:,:)      ! Left and right states for RP !
  REAL, POINTER      :: xFlux(:,:,:)           ! Flux in x direction          !
  REAL, POINTER      :: yFlux(:,:,:)           ! Flux in y direction          !
 
  !---------------------------------------------------------------------------!
  PUBLIC  :: iniTimeDisc, closeTimeDisc, TimeDisc
  !---------------------------------------------------------------------------!

CONTAINS

  SUBROUTINE IniTimeDisc(Const,Mesh)
    USE Types
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!
    ! Argument list declaration
    TYPE(tConstants)        :: Const
    TYPE(tMesh)             :: Mesh
    ! Local variable declaration
    INTEGER                 :: allocStat
    !---------------------------------------------------------------------------!

    ALLOCATE(k1(0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar),        &
             k2(0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar),        &
             xGradient(0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar), &
             yGradient(0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar), &
             xPrimLR(2,0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar),&
             yPrimLR(2,0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar),&
             xFlux(0:Mesh%IMAX,0:Mesh%JMAX,Const%nvar),         &
             yFlux(0:Mesh%IMAX,0:Mesh%JMAX,Const%nvar),         &
             new_cvar(0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar),  &
             STAT = allocStat )

    IF(allocStat.NE.0) THEN
       PRINT *, 'ERROR iniTimeDisc: Could not allocate all variables. '
       STOP
    ENDIF

    xGradient(:,:,:) = 0.
    yGradient(:,:,:) = 0.
    xFlux(:,:,:)     = 0.
    yFlux(:,:,:)     = 0.
    k1(:,:,:)        = 0.
    k2(:,:,:)        = 0.

  END SUBROUTINE IniTimeDisc

  SUBROUTINE CloseTimeDisc()
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!

    DEALLOCATE(k1,        &
               k2,        &
               xGradient, &
               yGradient, &
               xPrimLR,  &
               yPrimLR,  &
               xFlux,     &
               yFlux,     &
               new_cvar )

  END SUBROUTINE CloseTimeDisc

  SUBROUTINE TimeDisc(iter,dt,pvar,cvar,Const,Mesh,MPI)
    USE Types
    USE SetBoundaryCondition_mod
    USE ComputeGradients_mod
    USE Reconstruction_mod
    USE FluxCalculation_mod
    USE FluxBalance_mod
    USE PrimCons_mod
    !---------------------------------------------------------------------------!
    IMPLICIT NONE
    !---------------------------------------------------------------------------!
    ! Argument list declaration
    TYPE(tConstants)        :: Const
    TYPE(tMesh)             :: Mesh
    TYPE(tMPI)              :: MPI
    REAL                    :: dt
    REAL                    :: pvar(0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    REAL                    :: cvar(0:Mesh%IMAX+1,0:Mesh%JMAX+1,Const%nvar)
    INTEGER                 :: iter
    ! Local variable declaration
    !---------------------------------------------------------------------------!
    INTENT(INOUT)           :: pvar, cvar
    INTENT(IN)              :: Const, Mesh
    !---------------------------------------------------------------------------!

    CALL SetBoundaryCondition(pvar,Const,Mesh,MPI)
    IF(Const%SpaceOrder.EQ.2) THEN
       CALL ComputeGradients(xGradient,yGradient,pvar,Const,Mesh)
    ENDIF
    CALL Reconstruction(dt,xPrimLR,yPrimLR,xGradient,yGradient,pvar,Const,Mesh,MPI)
    CALL FluxCalculation(iter,dt,xFlux,yFlux,xPrimLR,yPrimLR,Const,Mesh,MPI)
    CALL FluxBalance(k1,xFlux,yFlux,Const,Mesh)

    ! Timeupdate from n to n+1 for all cells in conservative variables
    cvar(:,:,:) = cvar(:,:,:) + dt*k1(:,:,:)
    
  END SUBROUTINE TimeDisc


END MODULE TimeDisc_mod
