!*****************************************************************************!
!* 
!*      $RCSfile: TypesDef.f90,v $
!*
!*      $Revision: 1.5 $
!*
!*      $Date: 2004/02/09 11:52:16 $
!*
!*      $Author: roller $
!*
!*****************************************************************************!

MODULE Types
  !---------------------------------------------------------------------------!
  IMPLICIT NONE
  PRIVATE
  !---------------------------------------------------------------------------!
  TYPE tMesh
     INTEGER              :: IMAX, JMAX              ! Number of cells in i/j !
     REAL                 :: x0, y0, xlen, ylen      ! Computational domain   !
     REAL                 :: xNormal(2)              ! x-Side normal          !
     REAL                 :: yNormal(2)              ! y-Side normal          !
     REAL                 :: dx                      ! x-Side lengths in 2D   !
     REAL                 :: dy                      ! y-Side lengths in 2D   !
     REAL                 :: lmin                    ! Minimum side length    !
     REAL, POINTER        :: Bary(:,:,:)             ! Cell barycenters       !
     REAL, POINTER        :: Surface(:,:)            ! Cell surface in 2D     !
  END TYPE tMesh

  TYPE tConstants
     REAL                 :: Pi                      ! Pi                     !
     REAL                 :: gamma                   ! Ratio of spec. heats   !
     REAL                 :: CFL                     ! CFL number             !
     INTEGER              :: MaxIter                 ! No. of max. iterations !
     REAL                 :: Stoptime                ! Simulation endtime     !
     INTEGER              :: icType                  ! Type of initial cond.  !
     INTEGER              :: nvar                    ! Number of variables    !
     INTEGER              :: TimeOrder               ! Order of time integr.  !
     INTEGER              :: SpaceOrder              ! Order of space disc.   !
     INTEGER              :: iFlux                   ! Numerical scheme       !
     INTEGER              :: BoundCond               ! Boundary Condition     !
     ! Constants concerning initial condition
     REAL                 :: amplitude               ! Gausspulse amplitude   !
     REAL                 :: hwidth                  ! Gausspulse halfwidth   !
     REAL                 :: rhol,ul,vl,pl           ! RP problem left state  !
     REAL                 :: rhor,ur,vr,pr           ! RP problem left state  !
  END TYPE tConstants

  TYPE tFileIO
     INTEGER              :: uOutFile                ! unit of outputfile     !
     CHARACTER(LEN=60)    :: strOutFile              ! string of filename     !
     REAL                 :: iOutput                 ! interval of output     !
     INTEGER              :: iVisuProg               ! Tecplot or IBM DX      !
  END TYPE tFileIO

  TYPE tMPI
     integer              :: ierror                  ! Fehlerflag fr MPI     !
     integer              :: myrank                  ! Welcher Proze bin ich?!
     integer              :: myrank2                 ! Welcher kartes. Proze bin ich?!
     integer              :: mpisize                 ! Wieviel Prozesse gibt's!
     integer              :: comm_cart               ! Neuer Communicator     !
     integer              :: dims(2)                 ! #Prozesse pro Richtung !
     integer              :: coords(2)               ! Kartesische ProzessNr. !
     integer              :: xNumber,yNumber         ! Hilfsgre fr Zellverteilung
     integer              :: IMAX, JMAX              ! Gesamt-IMAX,JMAX       !
     integer              :: xOffset, yOffset        ! Wo im Rechengebiet bin ich?
     logical              :: periods(2)              ! Periodische Prozesse   !
  END TYPE tMPI
 

  !---------------------------------------------------------------------------!
  PUBLIC  :: tMesh, tConstants, tFileIO, tMPI
  !---------------------------------------------------------------------------!

  ! ==========================================================================!
  !                            Information on tMesh                           !
  ! ==========================================================================!
  !                                                                           !
  !       j/i     0     1     2     3     4     5     6     7     8           !
  !                  .     .     .     .     .     .     .     .              !
  !  y-Sides         .     .     .     .     .     .     .     .              !
  !              5   .     .     .     .     .     .     .     .              !
  !    4        .....o-----o-----o-----o-----o-----o-----o-----Z.....         !
  !                  |     |     |     |     |     |     |     |              !
  !              4   |     |     |     |     |     |     |     |              !
  !    3        .....o-----o-----o-----o-----o-----o-----o-----o.....         !
  !                  |     |     |     |     |     |     |     |              !
  !              3   |     |     |     |     |     |     |     |              !
  !    2        .....o-----o-----o-----o-----o-----o-----o-----o.....         !
  !                  |     |     |     |     |     |     |     |              !
  !              2   |     |     |     |     |     |     |     |              !
  !    1        .....o-----o-----o-----o-----o-----o-----o-----o.....         !
  !                  |     |     |     |     |     |     |     |              !
  !              1   |     |     |     |     |     |     |     |              !
  !    0        .....X-----o-----o-----o-----o-----o-----o-----o.....         !
  !                  .     .     .     .     .     .     .     .              !
  !              0   .     .     .     .     .     .     .     .              !
  !                  .     .     .     .     .     .     .     .              !
  ! x-Sides:         0     1     2     3     4     5     6     7              !
  !                                                                           !
  ! JMAX=4 / IMAX=7                : Number of cells in computational domain  !
  ! j=JMAX+1, j=0 / i=IMAX+1, i=0  : Ghost-cells                              !
  ! X = (x0/y0)                    : Absolute coordinate system origin        !
  ! Z = (x0+xlen,y0+ylen)          : Upper right corner of comp. domain       !
  !                                                                           !
  ! Details in cell (i,j)                                                     !
  !                                                                           !
  !                          ^                                                !
  !               ySide i,j  |                                                !
  !              o------------------------o                                   !
  !              |                        |                                   !
  !              |                        |                                   !
  !              |                        |                                   !
  !              |                        |                                   !
  !  xSide i-1,j |                        | xSide i,j                         !
  !              | --->      BC           | --->                              !
  !              |          (i,j)         |                                   !
  !              |                        |                                   !
  !              |                        |                                   !
  !              |           ^            |                                   !
  !              |           |            |                                   !
  !              o------------------------o                                   !
  !               ySide i,j-1                                                 !
  !                                                                           !
  ! Bary(i,j,1)       x-coordinate of barycenter BC                           !
  ! Bary(i,j,2)       y-coordinate of barycenter BC                           !
  ! Surface(i,j)      Cell surface                                            !
  ! dx                Side lengths of the left and right cell side            !
  ! dy                Side lengths of the upper and lower cell side           !
  ! xNormal(:)        Normal vector of left cell side                         !
  ! yNormal(:)        Normal vector of upper cell side                        !
  !                                                                           !
  !---------------------------------------------------------------------------!
  

! CONTAINS

END MODULE Types
